'use strict';

const um = require('unique-model');
const model = um.model;
const types = um.type;
const MySQLSession = require('../src').Session;

const databaseConfig = {
  host: 'localhost',
  database: 'um_sample_db',
  user: 'um_user',
  password: '123456',
};

const UInteger = types.UInteger;
const UDouble = types.UDouble;
const UBoolean = types.UBoolean;
const UString = types.UString;
const UObject = types.UObject;
const UObjectArray = types.UObjectArray;
const UDateTime = types.UDateTime;

let session = null;
let Privilege = null;
let privilegeDao = null;

async function main() {
  session = await um.createSession({
    backend: MySQLSession,
    options: databaseConfig
  });
  console.log('Create session successfully');

  Privilege = model.createModel('Privilege', {
    name: UString(),
    description: UString(),
    createdAt: UDateTime(),
    updatedAt: UDateTime()
  }, 'privileges');
  privilegeDao = session.getDao(Privilege);

  await removeAllPrivileges();
  const privileges = await createPrivileges();

  console.log(await Privilege.$extractArray(privileges, {
    recursive: true
  }));
  console.log(typeof privileges[0].id);

  session.close();
}

main().catch(err => {
  console.error('Error: ', err);
});

async function createPrivileges() {
  const privileges = [new Privilege({
    name: 'CreateUser',
    description: 'Create users',
    createdAt: new Date(),
    updatedAt: new Date()
  }), new Privilege({
    name: 'DeleteUser',
    description: 'Delete users',
    createdAt: new Date(),
    updatedAt: new Date()
  })];

  for ( const privilege of privileges ) {
    await privilegeDao.create(privilege);
  }

  return privileges;
}

async function removeAllPrivileges() {
  await privilegeDao.remove({});
}
