'use strict';

const { UpdateOperators } = require('../op/update');

function generateSetExpressions(updater) {
  const operatorKeys = Object.keys(updater);
  const setters = [];
  const values = [];

  for ( const operatorKey of operatorKeys ) {
    if ( !UpdateOperators.has(operatorKey) ) {
      throw new Error(`Unsupported update operator: ${operatorKey}`);
    }

    const updateContent = updater[operatorKey];
    const operator = UpdateOperators.get(operatorKey);

    operator.generator(updateContent, setters, values);
  }

  return {
    keyword: 'SET',
    sql: setters.join(', '),
    values: values
  };
}

module.exports = {
  generateSetExpressions
};
