'use strict';

const UpdateOperators = new Map([
  ['$set', {
    generator: generateSetUpdater
  }],
  ['$currentDate', {
    generator: generateCurrentDateUpdater
  }],
  ['$inc', {
    generator: generateIncUpdater
  }],
  ['$mul', {
    generator: generateMulUpdater
  }],
  ['$max', {
    generator: generateMaxUpdater
  }],
  ['$min', {
    generator: generateMinUpdater
  }]
]);

function generateSetUpdater(updateContent, setters, values) {
  const fieldKeys = Object.keys(updateContent);

  for ( const fieldKey of fieldKeys ) {
    const fieldValue = updateContent[fieldKey];
    setters.push(`${fieldKey} = ?`);
    values.push(fieldValue);
  }
}

function generateCurrentDateUpdater(updateContent, setters, values) {
  const fieldKeys = Object.keys(updateContent);

  for ( const fieldKey of fieldKeys ) {
    setters.push(`${fieldKey} = CURRENT_TIMESTAMP()`);
  }
}

function generateIncUpdater(updateContent, setters, values) {
  const fieldKeys = Object.keys(updateContent);

  for ( const fieldKey of fieldKeys ) {
    const fieldValue = updateContent[fieldKey];
    setters.push(`${fieldKey} = ${fieldKey} + ?`);
    values.push(fieldValue);
  }
}

function generateMulUpdater(updateContent, setters, values) {
  const fieldKeys = Object.keys(updateContent);

  for ( const fieldKey of fieldKeys ) {
    const fieldValue = updateContent[fieldKey];
    setters.push(`${fieldKey} = ${fieldKey} * ?`);
    values.push(fieldValue);
  }
}

function generateMaxUpdater(updateContent, setters, values) {
  const fieldKeys = Object.keys(updateContent);

  for ( const fieldKey of fieldKeys ) {
    const fieldValue = updateContent[fieldKey];
    setters.push(`${fieldKey} = IF(? > ${fieldKey}, ?, ${fieldKey})`);
    values.push(fieldValue);
    values.push(fieldValue);
  }
}

function generateMinUpdater(updateContent, setters, values) {
  const fieldKeys = Object.keys(updateContent);

  for ( const fieldKey of fieldKeys ) {
    const fieldValue = updateContent[fieldKey];
    setters.push(`${fieldKey} = IF(? > ${fieldKey}, ?, ${fieldKey})`);
    values.push(fieldValue);
    values.push(fieldValue);
  }
}

module.exports = {
  UpdateOperators
};
